indexing

	description: "Class that implements the portable file descriptor routines in POSIX."

	author: "Berend de Boer"
	date: "$Date: 2006/06/01 $"
	revision: "$Revision: #1 $"


class

	EPX_FILE_DESCRIPTOR


inherit

	ABSTRACT_FILE_DESCRIPTOR
		redefine
			is_blocking_io,
			set_blocking_io,
			supports_nonblocking_io
		end

	POSIX_BASE

	PAPI_UNISTD

	PAPI_FCNTL


creation

	make,
	open,
	open_read,
	open_write,
	open_read_write,
	open_truncate,
	create_read_write,
	create_write,
	create_with_mode,
	make_as_duplicate,
	attach_to_fd


feature -- non-blocking i/o

	is_blocking_io: BOOLEAN is
			-- Is blocking i/o enabled (default)?
		do
			Result := not test_bits (file_status_flags, O_NONBLOCK)
		end

	set_blocking_io (enable: BOOLEAN) is
			-- Set `is_blocking_io'.
		local
			new_flags: INTEGER
		do
			new_flags := flip_bits (file_status_flags, O_NONBLOCK, not enable)
			safe_call (posix_fcntl_arg (value, F_SETFL, new_flags))
		end

	supports_nonblocking_io: BOOLEAN is True


feature {NONE} -- Access

	file_status_flags: INTEGER is
			-- All status bits associated with this handle
		require
			open: is_open
		do
			Result := posix_fcntl (value, F_GETFL)
		end


feature {NONE} -- Implementation

	make_status is
		do
			create {POSIX_STATUS_FILDES} my_status.make (Current)
		end


feature {NONE} -- Abstract API binding

	abstract_close (fildes: INTEGER): INTEGER is
			-- Closes a a file
		do
			Result := posix_close (fildes)
		end

	abstract_create (a_path: POINTER; oflag, mode: INTEGER): INTEGER is
			-- Creates a new file or rewrites an existing one
			-- does not call `creat' but `open'!
		do
			Result := posix_create (a_path, oflag, mode)
		end

	abstract_dup (fildes: INTEGER): INTEGER is
			-- Duplicates an open file descriptor
		do
			Result := posix_dup (fildes)
		end

	abstract_dup2 (fildes, fildes2: INTEGER): INTEGER is
			-- Duplicates an open file descriptor
			-- fildes is the file descriptor to duplicate, fildes2 is the
			-- desired new file descriptor.
		do
			Result := posix_dup2 (fildes, fildes2)
		end

	abstract_isatty (fildes: INTEGER): BOOLEAN is
			-- Determines if a file descriptor is associated with a terminal
		do
			Result := posix_isatty (fildes)
		end

	abstract_lseek (fildes: INTEGER; offset, whence: INTEGER): INTEGER is
			-- Repositions read/write file offset
		do
			Result := posix_lseek (fildes, offset, whence)
		end

	abstract_open (a_path: POINTER; oflag: INTEGER): INTEGER is
			-- Opens a file
		do
			Result := posix_open (a_path, oflag)
		end

	abstract_read (fildes: INTEGER; buf: POINTER; nbyte: INTEGER): INTEGER is
			-- Reads from a file.
		do
			Result := posix_read (fildes, buf, nbyte)
		end

	abstract_write (fildes: INTEGER; buf: POINTER; nbyte: INTEGER): INTEGER is
			-- Reads from a file.
		do
			Result := posix_write (fildes, buf, nbyte)
		end


feature {NONE} -- error codes

	abstract_EWOULDBLOCK: INTEGER is
			-- The process would be delayed in the I/O operation
		do
			-- For Unix we assume EAGAIN and EWOULDBLOCK are equal.
			Result := EAGAIN
		end

	abstract_EINTR: INTEGER is
			-- read/write was interrupted
		do
			Result := EINTR
		end


feature {NONE} -- open constants

	abstract_O_APPEND: INTEGER is
			-- Set the file offset to the end-of-file prior to each write
		do
			Result := O_APPEND
		end

	abstract_O_CREAT: INTEGER is
			-- If the file does not exist, allow it to be created. This
			-- flag indicates that the mode argument is present in the
			-- call to open.
		do
			Result := O_CREAT
		end

	abstract_O_RDONLY: INTEGER is
			-- Open for reading only
		do
			Result := O_RDONLY
		end

	abstract_O_RDWR: INTEGER is
			-- Open fo reading and writing
		do
			Result := O_RDWR
		end

	abstract_O_TRUNC: INTEGER is
			-- Use only on ordinary files opened for writing. It causes
			-- the file to be truncated to zero length.
		do
			Result := O_TRUNC
		end

	abstract_O_WRONLY: INTEGER is
			-- Open for writing only
		do
			Result := O_WRONLY
		end


feature {NONE} -- permission constants

	abstract_S_IREAD: INTEGER is
		do
			Result := S_IREAD
		end

	abstract_S_IWRITE: INTEGER is
		do
			Result := S_IWRITE
		end


end
